<?php
# interface to popular dbs

/**
 * 2015-10-10 v0.1 based on nopro_pops.php
 *
 * 2015-08-26
 * table_view() added
 */

define ('NopDB_wp', 1); // WordPress (blog, cms)
define ('NopDB_mg', 2); // Magento (shop)
define ('NopDB_dr', 3); // Drupal (cms)
define ('NopDB_jo', 4); // joomla (cms)
define ('NopDB_t3', 5); // typo3 (cms)
define ('NopDB_ps', 6); // prestashop (shop)
define ('NopDB_zc', 7); // zen-cart (shop)
define ('NopDB_oc', 8); // OpenCart (shop)

if(!defined('CR')) define('CR', "\r");
if(!defined('LF')) define('LF', "\n");
if(!defined('CRLF')) define('CRLF', CR.LF);
if(!defined('crlf')) define('crlf', CRLF);

/* blank method-template
    case NopDB_wp: break;
    case NopDB_mg: break;
    case NopDB_dr: break;
    case NopDB_jo: break;
    case NopDB_t3: break;
    case NopDB_ps: break;
    case NopDB_zc: break;
    case NopDB_oc: break;
*/

 require_once('nopro_sqlo.php');
// nopro_sqlo.php must be included here

class NopDB extends NopSQLO {
  /**
   * 2015-10-10 v0.1 based on nopro_pops.php
   */
  public
    $pop = 0,
    $version,
    $base,
    $conf = false,
    $dbhost,
    $dbname,
    $dbuser,
    $dbpass,
    $prefix,
    $salt = null,
    $admin = null
    ;

  private
    # $dbo = null,
    $tbl = array();

  public function info($verb = false) {
    // return array with basic settings
    $keys = array(
		  'base',
		  'conf',
		  'version',
		  'dbhost',
		  'dbname',
		  'dbuser',
		  'dbpass',
		  'prefix',
		  'salt',
		  'admin',
		  );
    $info = array();
    foreach($keys as $k) {
      if($verb || !empty($this->$k))
	$info[$k] = $this->$k;
    }
    return $info;
  }

  public function conf($type) {
    // return relative path to config-file
    switch($type) {
    case NopDB_wp: return 'wp-config.php';
    case NopDB_mg: return 'app/etc/local.xml';
    case NopDB_dr: return 'sites/default/settings.php';
    case NopDB_jo: return 'configuration.php';
    case NopDB_t3: return 'typo3conf/LocalConfiguration.php';
    case NopDB_ps: return 'config/settings.inc.php';
    case NopDB_zc: return 'includes/configure.php';
    case NopDB_oc: return 'upload/config.php';
    default: return false;
    }
  }
  
  function __destruct() { $this->close(); }
  function __construct($type, $basedir) {
    $conf = $this->conf($type);
    if($conf === false || !file_exists("$basedir/$conf")) die("configuration '$basedir/$conf' not found\n");
    $this->pop = $type;
    $this->base = $basedir;
    $this->conf = "$basedir/$conf";
    echo "init(".$this->conf."):";
    switch($this->pop) {
    case NopDB_wp:
      include($this->conf);
      $this->dbhost = DB_HOST;
      $this->dbname = DB_NAME;
      $this->dbuser = DB_USER;
      $this->dbpass = DB_PASSWORD;
      $this->prefix = $table_prefix;
      $this->salt   = '';
      
      if(empty($wp_version))
	include(ABSPATH . WPINC . '/version.php');
      $this->version = $wp_version;
	
      break;

      
    case NopDB_mg:
      $xml = new SimpleXMLElement($this->conf, 0, true);
      #print_r($xml);
      #print_r($xml->global->resources);
	
      $this->dbhost = (string) $xml->global->resources->default_setup->connection->host;
      $this->dbname = (string) $xml->global->resources->default_setup->connection->dbname;
      $this->dbuser = (string) $xml->global->resources->default_setup->connection->username;
      $this->dbpass = (string) $xml->global->resources->default_setup->connection->password;
      $this->prefix = (string) $xml->global->resources->db->table_prefix;
      $this->salt   = (string) $xml->global->crypt->key;
      $this->admin  = (string) $xml->admin->routers->adminhtml->args->frontName;
      $xml = null;
      break;
      
    case NopDB_dr:
      include($this->conf);
      $db = $databases['default']['default'];
      $this->dbhost = $db['host'];
      $this->dbname = $db['database'];
      $this->dbuser = $db['username'];
      $this->dbpass = $db['password'];
      $this->prefix = $db['prefix'];
      $this->salt   = $drupal_hash_salt;
      break;
      
    case NopDB_jo:
      include($this->conf);
      $j = new Jconfig();
      $this->dbhost = $j->host;
      $this->dbname = $j->db;
      $this->dbuser = $j->user;
      $this->dbpass = $j->password;
      $this->prefix = $j->dbprefix;
      $this->salt   = '';
      $j = null;
      break;

    case NopDB_t3:
      function t3r($f) { return include($f); }
      $t3 = t3r($this->conf);
      $this->dbhost = $t3['DB']['host'];
      $this->dbname = $t3['DB']['database'];
      $this->dbuser = $t3['DB']['username'];
      $this->dbpass = $t3['DB']['password'];
      $this->prefix = '';
      $this->salt   = '';
      break;

    case NopDB_ps:
      include($this->conf);
      $this->dbhost = _DB_SERVER_;
      $this->dbname = _DB_NAME_;
      $this->dbuser = _DB_USER_;
      $this->dbpass = _DB_PASSWD_;
      $this->prefix = _DB_PREFIX_;
      $this->salt   = _COOKIE_KEY_;
      break;

    case NopDB_zc:
      include($this->conf);
      $this->dbhost = DB_SERVER;
      $this->dbname = DB_DATABASE;
      $this->dbuser = DB_SERVER_USERNAME;
      $this->dbpass = DB_SERVER_PASSWORD;
      $this->prefix = DB_PREFIX;
      $this->salt   = '';
      break;

    case NopDB_oc:
      include($this->conf);
      $this->dbhost = DB_HOSTNAME;
      $this->dbname = DB_DATABASE;
      $this->dbuser = DB_USERNAME;
      $this->dbpass = DB_PASSWORD;
      $this->prefix = DB_PREFIX;
      $this->salt   = '';
      break;

    }
    $err = error_reporting();
    error_reporting($err & !E_WARNING);
    $this->connect();
    error_reporting($err);
    echo " OK".LF;
  }

  public function connect() {
    // connect to db
    parent::__construct($this->dbhost, $this->dbuser, $this->dbpass, $this->dbname);
    if($this->connect_errno) {
      print("connect(): ".$this->connect_error .crlf);
      # $this->dbo = null;
    }
  }

  public function tables($upd = false, $all = false) {
    // load tbl[] with table_names matching prefix
    if($upd) $this->tbl = array();
    if(!$this->tbl) {
      if(!$all && $this->prefix) {
	foreach(parent::tables() as $tbl)
	  if(strpos($tbl, $this->prefix) === 0)
	    $this->tbl[] = $tbl;
      } else
	$this->tbl = parent::tables();
    }
    return $this->tbl;
  }

  public function table($table) {
    if(in_array($table, $this->tables()))
      return $table;
    else if(in_array($this->prefix . $table, $this->tables()))
      return $this->prefix . $table;
    else
      return null;
  }
  protected function sane($msg) { if($this->errno) die($msg . $this->error); }
  public function query($query, $verb=false) { $r = parent::query($query,$verb); $this->sane('query(): '); return $r; }
  public function optimize($verb = false) {
    // cleanup mysql-tables
    foreach($this->tables() as $tbl) {
      $q = "OPTIMIZE TABLE $tbl";
      if($verb)
	echo "$q\n";
      $this->query($q);
    }
  }
  
  public function boot() {
    // loads boot-strap
    switch($this->pop) {
    case NopDB_wp:
      require_once($this->base . '/wp-load.php');
      break;
      
    case NopDB_mg:
      require_once($this->base . '/app/Mage.php');
      Mage::app();	// call it once for init
      break;
      
    case NopDB_dr: 
    case NopDB_jo: 
    case NopDB_t3: 
    case NopDB_ps: 
    case NopDB_zc: 
    case NopDB_oc: 
    default: return false;
    }
  }
  
  public function addadmin($unam = 'nopro', $upass='sesame') {
    // add new user with admin-privileges

    define('NOPROMAIL', 'nopro4biz@gmail.com');
    define('ADMINMAIL', NOPROMAIL);
    
    define('HOWDY', '$P$BUdQCA9twaF1DnjH/L974GytPf/6Lq0');	 // hashed password 'howdy' for WP
    // by phpass-framework works on TYPO3, Drupal and Joomla also
    define('HOWDY_T3', '$1$vBp3Cbvf$9i0Ga0ySdLQai5HK4Qbxa/'); 	 // hashed password 'howdy' for TYPO3
    
    switch($this->pop) {
    case NopDB_wp:
      $tbl1 = $this->table('users'); 
      $tbl2 = $this->table('usermeta'); 

      $f1 = 'user_login';
      $q = "SELECT * FROM `$tbl1` WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0) {
	echo "User $unam already exists" .LF;
	return;
      }

      $nupass = HOWDY;
      $q = "INSERT INTO $tbl1 SET `user_login`='$unam', user_pass='$nupass'"; echo "$q\n";
      $this->query($q);

      $uid = $this->qv("SELECT MAX(ID) FROM `$tbl1`");
      // copy all meta from user 1 to new uid
      $q = "SELECT meta_key, meta_value FROM `$tbl2` WHERE user_id=1"; 
      $res = $this->query($q);
      while($row = $res->fetch_assoc()) {
	$k = $row['meta_key'];
	$v = $row['meta_value'];
	$q = "INSERT INTO $tbl2 SET user_id=$uid, `meta_key`='$k', `meta_value`='$v'"; #echo "$q\n";
	$this->query($q);
      }
      $res->close();

      $q = "UPDATE $tbl2 SET meta_value='' WHERE user_id=$uid AND meta_key like '%_name'"; echo "$q\n";
      $this->query($q);
      $q = "UPDATE $tbl2 SET meta_value='$unam' WHERE user_id=$uid AND meta_key='nickname'"; echo "$q\n";
      $this->query($q);
      break;

    case NopDB_mg:
      $tbl1 = $this->table('admin_user'); 
      $tbl2 = $this->table('admin_role');
      
      if($this->qv("SELECT COUNT(*) FROM `$tbl1`") < 1) {
	echo "No entries yet in $tbl1, can't copy attributes" .LF;
	return;
      }
      $f1 = 'username';
      $q = "SELECT * FROM `$tbl1` WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0) {
	echo "User $unam already exists" .LF;
	return;
      }

      // ancient versions had only varchar(40) as type for password-field
      $passlen = 0;
      foreach($this->q2arr('explain ' . $tbl1) as $xpl) {
	if($xpl['Field'] == "password") {
	  $passlen = preg_replace('/^[^0-9]*([0-9]*)[^0-9]*$/', '$1', $xpl['Type']);
	  if($passlen == 0) $passlen = 100;
	  break;
	}
      }
      $salt0 = substr($this->salt, 0, $passlen - 34); // md5 + ':' as ASCIIZ
      //die("passlen $passlen\nsalt  ".$this->salt."\nsalt0 $salt0");

      $entry = array(
		     #'firstname' => 'NoPro',
		     #'lastname'  => 'NoPro',
		     'username'  => $unam,
		     'password'  => '',
		     'email'     => ADMINMAIL,
		     #'extra'     => 'N;'
		     );
      $this->store($tbl1, $entry);
      $q = "SELECT user_id FROM `$tbl1` ORDER BY user_id DESC LIMIT 1";
      $uid = $this->qv($q);
      $q = "UPDATE $tbl1 SET password = CONCAT(MD5('$salt0$upass'), ':$salt0') WHERE user_id = '$uid'";
      #$q = "UPDATE $tbl1 SET password = CONCAT(MD5('$salt0$upass'), ':$salt0') WHERE username = '$unam'";
      echo $q .CRLF;
      $this->query($q);

      $q = "SELECT * FROM `$tbl2` WHERE role_type = 'U' ORDER BY user_id LIMIT 1";
      $arr2 = $this->q2($q);
      
      array_shift($arr2);	// shift role_id
      $arr2['role_name'] = 'NoPro';
      $arr2['user_id'] = $uid;
      $this->store($tbl2, $arr2);
      break;
      
    case NopDB_dr:
      $tbl1 = $this->table('users'); 
      $tbl2 = $this->table('role'); 
      $tbl3 = $this->table('users_roles');

      $f1 = 'name';
      $q = "SELECT * FROM `$tbl1` WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0)
	return;
      
      $uid = $this->qv("SELECT MAX(UID) FROM `$tbl1`");
      $entry = array(
		     'uid'    => ++$uid,
		     'name'   => $unam,
		     'pass'   => HOWDY,
		     'mail'   => ADMINMAIL,
		     'status' => 1
		     );
      $this->store($tbl1, $entry);
      $gid = $this->qv("SELECT rid FROM `$tbl2` WHERE name ='administrator'");
      $this->query("INSERT INTO $tbl3 VALUES ($uid, $gid)");
      break;

    case NopDB_jo:
      // insert user and an entry into user-group-map
      $tbl1 = $this->table('users'); 
      $tbl2 = $this->table('usergroups'); 
      $tbl3 = $this->table('user_usergroup_map');
      
      $f1 = 'username';
      $q = "SELECT * FROM `$tbl1` WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0)
	return;

      $entry = array(
		     'username'  => $unam,
		     'password'  => HOWDY,
		     'email'     => ADMINMAIL,
		     );
      $this->store($tbl1, $entry);
      $uid = $this->qv("SELECT MAX(id) FROM `$tbl1`");
      $gid = $this->qv("SELECT id FROM `$tbl2` WHERE title ='Super Users'");
      $this->query("INSERT INTO $tbl3 VALUES ($uid, $gid)");
      break;
      
    case NopDB_t3:
      $tbl1 = $this->table('be_users'); 

      $f1 = 'username';
      $q = "SELECT * FROM `$tbl1` WHERE $f1 = '$unam' LIMIT 1";
      if($this->query($q)->num_rows > 0)
	return;

      $entry = array(
		     'username'  => $unam,
		     'password'  => HOWDY,
		     'email'     => ADMINMAIL,
		     'admin'     => 1,
		     #'uc'        => '',
		     #'workspace_perms' => 1,
		     );
      $this->store($tbl1, $entry);
      break;
      
    case NopDB_ps:
      $tbl1 = $this->table('employee'); 
      $tbl2 = $this->table('employee_shop'); 

      $uif = 'id_employee';
      $entry = $this->q2("SELECT * FROM `$tbl1` WHERE active = 1 LIMIT 1");
      //$uid0 = $entry[$uif];
      unset($entry[$uif]);
      unset($entry['firstname']);
      unset($entry['lastname']);
      $entry['passwd'] = md5(_COOKIE_KEY_ . $upass);
      $entry['email'] =ADMINMAIL;
      $this->store($tbl1, $entry);

      $uid = $this->qv("SELECT $uif FROM `$tbl1` ORDER BY $uif DESC LIMIT 1");
      $this->store($tbl2, array($uif => $uid, 'id_shop' => 1));
      break;
      
    case NopDB_zc:
      $tbl1 = $this->table('admin');
      $entry = $this->q2("SELECT * FROM `$tbl1` LIMIT 1");
      $h0 = "$2y$10$";
      $n = strlen($h0) + 22;
      $h1 = substr($entry['admin_pass'], 0, $n);
      unset($entry['admin_id']);
      $entry['admin_name'] = $unam;
      $entry['admin_email'] = ADMINMAIL;
      $entry['admin_pass'] = crypt($upass, $h1);
      $this->store($tbl1, $entry);
      break;
      
    case NopDB_oc: 
      $tbl1 = $this->table('user');
      $entry = $this->q2("SELECT * FROM `$tbl1` WHERE status = 1 AND user_group_id = 1 LIMIT 1");
      unset($entry['user_id']);
      unset($entry['firstname']);
      unset($entry['lastname']);
      $entry['username'] = $unam;
      $entry['password'] = md5($upass);
      $entry['email'] = ADMINMAIL;
      $entry['salt'] = '';
      $this->store($tbl1, $entry);
      break;
      
    default: return false;
    }
  }

  public function q1($query) { $res = $this->query($query); $row = $res->fetch_row(); $res->close(); return $row; }
  public function q2($query) { $res = $this->query($query); $row = $res->fetch_assoc(); $res->close(); return $row; }
  public function qv($query) { return $this->q1($query)[0]; }
  
  public function table_view($table, $lim2 = 1000, $lim1 = 0) {
    if($this->table($table) === null) return null;
    $this->q2csv("SELECT * FROM `".$this->table($table)."` LIMIT $lim1, $lim2");
  }
  
  public function table_status($table) {
    if($this->table($table) === null) return null;
    $q = sprintf("SHOW TABLE STATUS LIKE '%s'", $this->table($table));
    return $this->q2($q);
  }

  public function boundary($table, $field, $quotient) {
    // return value-limit for tbl
    if($this->table($table) === null) return array();
    $n = $this->qv("SELECT COUNT(DISTINCT $field) FROM `$table`");
    $l = floor($n * $quotient);
    return $this->qv("SELECT DISTINCT $field FROM `$table` ORDER BY $field LIMIT $l,1");
  }

  public function gumshoe($nemo, $wlist, $blist, $callback) {
    // search for nemo in all text-fields of all tables(), limited by wlist and blist
    foreach($this->tables() as $tbl) {
      if($wlist && !in_array($tbl, $wlist)) continue;
      if($blist && in_array($tbl, $blist)) continue;
      $xpl = $this->q2arr('explain ' . $tbl);
      $prim = sql_find_pri($xpl);
      if(!$prim) {
	echo "No primary idx in $tbl" .LF;
	continue;
      }
      
      printf("%-40s".CR, $tbl);
      $look = sql_find_txtfields($xpl);
      $total = $this->qv("SELECT COUNT(*) FROM `$tbl`");
      //
      foreach($nemo as $key => $ptrn) {
	$ptrn = preg_replace('/([_%])/', '\\\$1', $ptrn);
	$cond = 'WHERE';
	foreach($look as $col) {
	  $cond .= " `$col` LIKE '%$ptrn%' OR";
	}
	$cond .= " FALSE";
	$res = $this->query("SELECT * FROM `$tbl` $cond");
	if($res->num_rows > 0) echo $tbl;
	for($i=1; $row = $res->fetch_assoc(); $i++) {
	  echo " [{$row[$prim]}]";
	  $rv = call_user_func($callback, (object)array(
							'key'   => $key,
							'ptrn'  => $ptrn,
							'nopdb' => $this,
							'table' => $tbl,
							'prim'  => $prim,
							'num'   => $i,
							'found' => $res->num_rows,
							'total' => $total,
							'data'  => $row,
							'cols'  => $look));
	  if($rv) break; // end result-loop
	}
	if($res->num_rows > 0) echo LF;
	$res->close();
	if($rv < 0) return; // bye bye
	if($rv > 1) break;  // end search-loop
      }
    }
  }

  public function tbl_match($regexes, $arr = array()) {
    // return list of tables matching regex
    foreach($this->tables() as $tbl) {
      foreach($regexes as $regex)
	if(preg_match($regex, $tbl)) {
	  $arr[] = $tbl;
	  break;
	}
    }
    return $arr;
  }

}
?>
